/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2009  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_H8S2378.c   (for Hitachi H8S2378 and HEW compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/


#ifndef   OS_FSYS                   /* CPU main clock frequncy 16 MHz x 2 (PLL)  */
  #define OS_FSYS 16000000uL*2
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */ 
  #define OS_PCLK_TIMER OS_FSYS     /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART OS_FSYS      /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ  (1000)
#endif
  
/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/
#ifndef   OS_UART
  #define OS_UART 1       
#endif    
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

#define OS_UART_USED (OS_UART == 1)

#define OS_TIMER_RELOAD ((OS_PCLK_TIMER/OS_TICK_FREQ)-1)
#if (OS_TIMER_RELOAD >= 0x10000)
  #error "TPU Timer can not be used, please check configuration"
#endif

/*********************************************************************
*
*       SFR definition of sfrs used in RTOSInit.c
*
*       We define sfrs here to be independend of any i/o header file
*/
#define _PLLCR       (*(volatile OS_U8 *) (0xFFFF45))  /* System Control            */
#define _SCKCR       (*(volatile OS_U8 *) (0xFFFF3B))  /* System Control            */
#define _SYSCR       (*(volatile OS_U8 *) (0xFFFF3D))  /* System Control            */
#define _MSTPCRH     (*(volatile OS_U8 *) (0xFFFF40))  /* System Control            */
#define _MSTPCRL     (*(volatile OS_U8 *) (0xFFFF41))  /* System Control            */
#define _INTCR       (*(volatile OS_U8 *) (0xFFFF31))  /* System Control            */
#define _IPRF        (*(volatile OS_U16*) (0xFFFE0A)) /* interrupt priority reg.F  */
#define _IPRG        (*(volatile OS_U16*) (0xFFFE0C)) /* interrupt priority reg.G  */
#define _IPRI        (*(volatile OS_U16*) (0xFFFE10)) /* interrupt priority reg.I  */
#define _IPRJ        (*(volatile OS_U16*) (0xFFFE12)) /* interrupt priority reg.J  */

#define _TPU_TSTR    (*(volatile OS_U8 *) (0xFFFFC0))  /* Timer start               */
#define _TPU_TCR0    (*(volatile OS_U8 *) (0xFFFFD0))  /* Timer control       ch0   */
#define _TPU_TMDR0   (*(volatile OS_U8 *) (0xFFFFD1))  /* Timer mode register ch0   */
#define _TPU_TIOR0   (*(volatile OS_U8 *) (0xFFFFD2))  /* Timer I/O ctrl      ch0   */
#define _TPU_TIER0   (*(volatile OS_U8 *) (0xFFFFD4))  /* Timer int enable    ch0   */
#define _TPU_TSR0    (*(volatile OS_U8 *) (0xFFFFD5))  /* Timer status        ch0   */
#define _TPU_TCNT0   (*(volatile OS_U16*) (0xFFFFD6))  /* Timer count reg.    ch0   */
#define _TPU_TGRA0   (*(volatile OS_U16*) (0xFFFFD8))  /* Timer general regA  ch0   */

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW
*
*
*  Initialize hardware required for embOS to run.
*  1. We switch CPU into interrupt mode 2 (8 interrupt priorities)
*  2. We initialize a timer for embOS with low interrupt priority.
*     We use 16bit Counter 2 in output compare mode.
*     This function should work on any hardware, but may have to be modified,
*     if you decide to use an other timer.   
*/
void OS_InitHW(void) {
  OS_IncDI();                           // Initially disable interrupts
  //
  // Enable PLL
  //
  _SCKCR |= (1 << 3);                  // PLL changes affect immediately
  _PLLCR = 0x01;                       // PLL * 2     
  //
  // Select interrupt mode 
  //
  _INTCR = (2 << 4);                   // Select interrupt mode 2
  //
  // Init Timer TPU Channel 0
  //
  _MSTPCRH   &= ~(1 << 3);             // Release TPU stop mode
  _TPU_TSTR  &= ~(1 << 0);             // Stop counter
  _TPU_TCR0   =  (1 << 5);             // Prescaler set to 1 (b2..b0: 0), Clear TCNT on TRGA compare, Count on rising edge
  _TPU_TIOR0 &= 0xF0;                  // No output for TRGA compare match
  _TPU_TGRA0  = OS_TIMER_RELOAD;       // Set Timer reload value for 1 msec interrupts
  _TPU_TMDR0  = 0x00;                  // Set timer to normal mode
  _TPU_TIER0 |= (1 << 0);              // Enable compare match interrupt
  _IPRF       = 0x0010;                // Set priority to 1
  _TPU_TSTR  |= (1 << 0);              // Start counter
  OS_COM_Init();                       // Initialize communication to embOSView
  OS_DecRI();                          // Restore low priority interrupt enable state
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*  
*  Please note:
*  This is basically the "core" of the idle loop
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) { // Idle loop: No task is ready to exec
  for (;;);          // Nothing to do ... wait for an interrupt
}

/*********************************************************************
*
*       OS_GetTimeCycles()
*
*  This routine is required for task-info via embOSView
*  or high precision time measurment functions.
*  It returns the system time in clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 Cnt;
  OS_U32 Time;

  Cnt = _TPU_TCNT0;
  Time = OS_Time;
  if(_TPU_TSR0 & (1 << 0)) { // Re-read if interrupt pending
    Time++;
    Cnt = _TPU_TCNT0;
  }
  return (OS_PCLK_TIMER/1000) * Time + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*  
*  Purpose:
*     Convert Cycles into micro seconds.
*  
*     If your clock frequency is not a multiple of 1 MHz, you may have to
*     modify this routine in order to get proper diagonstics.
*  Please note:
*     This routine is required for profiling or high resolution time
*     measurement. It does not affect operation of embOS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_TickHandler()
*
*       embOS timer interrupt handler
*/
void OS_ISR_TickHandler(void) {
  OS_EnterIntStack();
  _TPU_TSR0 &= ~(1 << 0);  // reset pending flag
  OS_HandleTick();
  OS_LeaveIntStack();
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
*       embOS timer interrupt handler
*/
__interrupt(vect=40) void OS_ISR_Tick (void) {  
  OS_CallNestableISR(&OS_ISR_TickHandler);  
}    

/*********************************************************************
*
*       Communication for embOSView via UART
*
*/
#if OS_UART_USED

#define BRR_VAL ((OS_PCLK_UART+OS_BAUDRATE*16L)/(OS_BAUDRATE*32L)-1)
#if BRR_VAL > 0xFF
  #error "UART can not be used with standard intialization."  
#endif

#define _SCI1_SMR   (*(volatile OS_U8 *) (0xFFFF80)) /* SCI Serial mode register    */
#define _SCI1_BRR   (*(volatile OS_U8 *) (0xFFFF81)) /* SCI Bit rate register       */
#define _SCI1_SCR   (*(volatile OS_U8 *) (0xFFFF82)) /* SCI Serial control register */
#define _SCI1_TDR   (*(volatile OS_U8 *) (0xFFFF83)) /* SCI Transmit data register  */
#define _SCI1_SSR   (*(volatile OS_U8 *) (0xFFFF84)) /* SCI Serial status register  */
#define _SCI1_RDR   (*(volatile OS_U8 *) (0xFFFF85)) /* SCI Receive data register   */
#define _SCI1_SCMR  (*(volatile OS_U8 *) (0xFFFF86)) /* SCI Smart card moderegister */

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  _MSTPCRL    &=  ~(1 << 2);             // Enable SIO module (clear stop mode) 
  _SCI1_SCR   &= ~((1 << 5)              // Clear Tx enable
                  |(1 << 4));            // Clear Rx enable
  _SCI1_SCR   =   (0 << 7)               // Clear Tx interrupt enable
                 |(0 << 6)               // Clear Rx interrupt enable
                 |(0 << 5)               // Clear Tx enable
                 |(0 << 4)               // Clear Rx enable
                 |(0 << 3)               // Clear multi processor mode
                 |(0 << 2)               // Clear Tx end interrupt enable
                 |(0 << 1)               // Set internal clock mode
                 |(0 << 0);              // SCLK output not used
  _SCI1_SMR   =   (0 << 7)               // Select Async. mode
                 |(0 << 6)               // Select 8 data bits
                 |(0 << 5)               // Disable parity
                 |(0 << 4)               // Even parity (don't care)
                 |(0 << 3)               // Select 1 stop bit
                 |(0 << 2)               // Disable multi processor mode
                 |(0 << 1)               // Clock divisor 1
                 |(0 << 0);              // Clock divisor 1
  _SCI1_BRR   =  BRR_VAL;                // Set calculated divisor
  _IPRJ       =  0x1000;                 // Set UART int to lowest priority (1) 
  _SCI1_SCR  |=   (1 << 6)               // Enable Rx interrupt
                 |(1 << 5)               // Enable Tx
                 |(1 << 4);              // Enable Rx
}

/*********************************************************************
*
*       OS_COM_Send1() for UART0: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  _SCI1_TDR  = c;
  _SCI1_SSR &= ~(1 << 7);   // Reset pending flag
  _SCI1_SCR |= (1<<7);      //  Enable Tx-Interrupt  
}

/*********************************************************************
*
*       OS_ISR_Tx_Handler() 
*
*       Uart Tx Interrupt Handler
*/
void OS_ISR_Tx_Handler(void) {
  _SCI1_SSR &= ~(1 << 7); // Reset pending flag
;;  OS_EnterIntStack();
  if (OS_OnTx()) {      //  No more characters to send ?
    _SCI1_SCR &= ~(1<<7); //  Disable Tx Interrupt
  }
  OS_LeaveIntStack();
}

/*********************************************************************
*
*       OS_ISR_tx() 
*      
*       ISR for uart tx
*/
__interrupt(vect=94) void OS_ISR_tx (void) {
  OS_CallNestableISR(&OS_ISR_Tx_Handler);  
}

/*********************************************************************
*
*       OS_ISR_Rx_Handler()
*
*       Uart Rx Interrupt Handler
*/
void OS_ISR_Rx_Handler(void) {
  _SCI1_SSR &= ~(1 << 6); // Reset pending flag
  OS_EnterIntStack();
  OS_OnRx(_SCI1_RDR);  
  OS_LeaveIntStack(); 
}

/*********************************************************************
*
*       OS_ISR_rx() 
*
*       ISR for uart rx 
*/
__interrupt(vect=93) void OS_ISR_rx (void) {
  OS_CallNestableISR(&OS_ISR_Rx_Handler);  
}

/*********************************************************************
*
*       OS_ISR_Err_Handler()
*
*       Uart Error Interrupt Handler
*/
void OS_ISR_Err_Handler(void) {
  _SCI1_SSR &= ~((1<<5) | (1<<4) | (1<<3));
}

/*********************************************************************
*
*       OS_ISR_err() 
*
*       ISR for uart error interrupt
*/
__interrupt (vect=92) void OS_ISR_err (void) {
  OS_CallNestableISR(&OS_ISR_Err_Handler);
}

/*********************************************************************
*
*       Communication for embOSView not selected
*
*/
#else  /* No communication routines required, use dummies  */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }

#endif /* OS_UART_USED */

/*****  EOF  ********************************************************/
