/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2009  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_H8S2472.c   (for Renesas H8S2472 and HEW compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/


#ifndef   OS_FSYS                   /* CPU main clock frequncy 8.5 Mhz x 4 (PLL) = 34 MHz */
  #define OS_FSYS 8500000L*4
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */ 
  #define OS_PCLK_TIMER (OS_FSYS/2) /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART  OS_FSYS     /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ  (1000)
#endif
  
/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART0 per default
*       and supports UART0 (SCIF)
*/
#ifndef   OS_UART
  #define OS_UART 0      
#endif    
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

#define OS_UART_USED (OS_UART == 0)

#define OS_TIMER_RELOAD (OS_PCLK_TIMER/OS_TICK_FREQ)
#if (OS_TIMER_RELOAD >= 0x10000)
  #error "FRT Timer can not be used, please check configuration"
#endif

/*********************************************************************
*
*       SFR definition of sfrs used in RTOSInit.c
*
*       We define sfrs here to be independend of any i/o header file
*/
#define SYSCR      (*(volatile OS_U8  *)(0xFFFFC4))  /* System control register     */
#define MSTPCRH    (*(volatile OS_U8  *)(0xFFFF86))  /* Modul standby register high */
#define MSTPCRL    (*(volatile OS_U8  *)(0xFFFF87))  /* Modul standby register low  */
#define FRT_TIER   (*(volatile OS_U8  *)(0xFFFF90))  /* Timer Interrupt Enable      */
#define FRT_TCSR   (*(volatile OS_U8  *)(0xFFFF91))  /* Timer control/status        */
#define FRT_FRC    (*(volatile OS_U16 *)(0xFFFF92))  /* Free running count          */
#define FRT_OCRA   (*(volatile OS_U16 *)(0xFFFF94))  /* Output compare register A   */
#define FRT_TCR    (*(volatile OS_U8  *)(0xFFFF96))  /* Timer control register      */
#define FRT_TOCR   (*(volatile OS_U8  *)(0xFFFF97))  /* Timer Output compare        */

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW
*
*
*  Initialize hardware required for embOS to run.
*  1. We switch CPU into interrupt mode 1 
*  2. We initialize a timer for embOS with low interrupt priority.
*     We use 16bit FRT Counter in output compare mode.
*     This function should work on any hardware, but may have to be modified,
*     if you decide to use an other timer.   
*/
void OS_InitHW(void) {
  OS_IncDI();                         // Initially disable interrupts
  SYSCR    |= (1 << 4);               // Set Interrupt mode 1 
  MSTPCRH &= ~(1 << 5);               // Disable standby for FRT
  FRT_OCRA = OS_TIMER_RELOAD;         // Set compare register
  FRT_TCSR = 0x01;                    // Clear FRC at compare match
  FRT_TCR  = 0x00;                    // Internal clock source / 2
  FRT_TOCR = 0x00;                    // Output Compare A Mode Select
  FRT_TIER = 0x08;                    // Output Compare Interrupt A Enable
  OS_COM_Init();                      // Initialize communication to embOSView
  OS_DecRI();                         // Restore low priority interrupt enable state
}

/*********************************************************************
*
*       OS_ISR_Tick_Handler()
*
*       embOS timer interrupt handler
*/
void OS_ISR_Tick_Handler(void) {
  FRT_TCSR &= ~(1 << 3);              // Clear the interrupt flag 
  OS_HandleTick();
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
*       embOS timer interrupt handler
*/
__interrupt(vect=52) void OS_ISR_Tick (void) {  
  OS_CallISR(&OS_ISR_Tick_Handler);
}    

/*********************************************************************
*
*       OS_GetTimeCcycles()
*
*  This routine is required for task-info via embOSView
*  or high precision time measurment functions.
*  It returns the system time in clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 Cnt;
  OS_U32 Time;
  Cnt = FRT_FRC;
  Time = OS_Time;
  if (FRT_TCSR & (1 << 3)) { // Re-read if interrupt pending
    Time++;
    Cnt = FRT_FRC;
  }
  return (OS_PCLK_TIMER/1000) * Time + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*  
*  Purpose:
*     Convert Cycles into micro seconds.
*  
*     If your clock frequency is not a multiple of 1 MHz, you may have to
*     modify this routine in order to get proper diagonstics.
*  Please note:
*     This routine is required for profiling or high resolution time
*     measurement. It does not affect operation of embOS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/((OS_PCLK_TIMER)/1000000);
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*  
*  Please note:
*  This is basically the "core" of the idle loop
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) { // Idle loop: No task is ready to exec
  for (;;);          // Nothing to do ... wait for an interrupt
}

#if (OS_UART_USED)

#define SCIF_FRBR   (*(volatile OS_U8  *)(0xFFFC80))  /* Receive buffer           */
#define SCIF_FTHR   (*(volatile OS_U8  *)(0xFFFC80))  /* Transmit holding         */
#define SCIF_FDLL   (*(volatile OS_U8  *)(0xFFFC80))  /* Divisor latch L          */
#define SCIF_FIER   (*(volatile OS_U8  *)(0xFFFC81))  /* Interrupt enable         */
#define SCIF_FDLH   (*(volatile OS_U8  *)(0xFFFC81))  /* Divisor latch H          */
#define SCIF_FIIR   (*(volatile OS_U8  *)(0xFFFC82))  /* Interrupt identification */
#define SCIF_FFCR   (*(volatile OS_U8  *)(0xFFFC82))  /* FIFO control             */
#define SCIF_FLCR   (*(volatile OS_U8  *)(0xFFFC83))  /* Line control             */
#define SCIF_FMCR   (*(volatile OS_U8  *)(0xFFFC84))  /* Modem control            */
#define SCIF_FLSR   (*(volatile OS_U8  *)(0xFFFC85))  /* Line status              */
#define SCIF_FMSR   (*(volatile OS_U8  *)(0xFFFC86))  /* Modem Status             */
#define SCIF_FSCR   (*(volatile OS_U8  *)(0xFFFC87))  /* Scratch pad              */
#define SCIF_SCIFCR (*(volatile OS_U8  *)(0xFFFC88))  /* SCIF control             */
#define SUBMSTPBL   (*(volatile OS_U8  *)(0xFFFE3F))  /* Submodule stop control   */ 

// bit defines for SCIFCR
#define SCIFCR_SCIFOE1_BIT  7
#define SCIFCR_SCIFOE0_BIT  6
#define SCIFCR_OUT2LOOP_BIT 4
#define SCIFCR_CKSEL1_BIT   3
#define SCIFCR_CKSEL0_BIT   2
#define SCIFCR_SCIFRST_BIT  1
#define SCIFCR_REGRST_BIT   0
// bit defines for FLCR
#define FLCR_DLAB_BIT       7
// bit defines for FIER
#define FIER_ETBEI_BIT      1
#define FIER_ERBFI_BIT      0
// bit defines for FMCR
#define FMCR_OUT2_BIT       3
// bit defines for FLSR
#define FLSR_TEMT           6 
#define FLSR_THRE           5

#define UART_CLK_DIVIDER    11

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  SUBMSTPBL   &= ~(1 << 3);                    // Disable stop mode for SCIF
  SCIF_SCIFCR  = (1 << SCIFCR_CKSEL0_BIT);     // Select clock source divided by 11 as baudrate clock
  SCIF_SCIFCR |= (3 << SCIFCR_SCIFOE0_BIT);    // Enable P50 pin for TX, disable P27 and P65
  SCIF_FLCR   |= (1 << FLCR_DLAB_BIT);         // Enable access to FDLL and FDLH registers
  SCIF_FDLL    = OS_PCLK_UART / (OS_BAUDRATE * 16 * UART_CLK_DIVIDER); // Baud rate = (Clock frequency input to baud rate generator) / (16 x divisor value)
  SCIF_FLCR   &= ~(1 << FLCR_DLAB_BIT);        // Disable access to FDLL and FDLH registers
  SCIF_FLCR    = 0x03;                         // Set 1 stop bit, 8-bit data, no parity 
  SCIF_FIER   |= (1 << FIER_ERBFI_BIT);        // Enable receive interrupt
  SCIF_FMCR   |= (1 << FMCR_OUT2_BIT );        // Enables the SCIF interrupt
}

/*********************************************************************
*
*       OS_COM_Send1() for UART0: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  SCIF_FIER |= (1 << FIER_ETBEI_BIT);          // Enable transmit interrupt
  SCIF_FTHR  = c;
}

/*********************************************************************
*
*       OS_ISR_ICF_Handler
*
*/
void OS_ISR_ICF_Handler(void) {
  char c;
  c = (SCIF_FIIR & 0x0F) >> 1;                 // mask all bits except INTID 
  switch (c) {
    case 3:                                    // Overrun error, parity error, framing error, break interrupt
	  c = SCIF_FLSR;                             // Clear pending bit
	  break;	  
	case 2:                                      // Receive data ready
	  c = SCIF_FRBR;                             // Clear pending bit
	  OS_OnRx(SCIF_FRBR); 
	  break;
	case 1:                                      // FTHR empty
	  c = SCIF_FRBR;                             // Clear pending bit
	  if (OS_OnTx()) {                           // No more characters to send?
        SCIF_FIER &= ~(1 << FIER_ETBEI_BIT);   // Disable transmit interrupt
	  } 
	  break;	
	default:
	  break;  	    
  }
}

/*********************************************************************
*
*       OS_ISR_tx() 
*      
*       ISR for uart tx
*/
__interrupt(vect=92) void OS_ISR_SCIF (void) {
  OS_CallISR(&OS_ISR_ICF_Handler);
}

/*********************************************************************
*
*       Communication for embOSView not selected
*
*/
#else  /* No communication routines required, use dummies  */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }

#endif /* OS_UART_USED */

/*****  EOF  ********************************************************/
