/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2009  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_H8SX1668R.c (for Renesas H8SX1668R and HEW)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include "iodefine.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef   OS_FSYS                   /* CPU main clock frequency     */
  #define OS_FSYS 12000000uL*4
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */ 
  #define OS_PCLK_TIMER OS_FSYS/2   /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART OS_FSYS/2    /* Normally derived from CPU    */
#endif                              /* clock                        */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ  (1000)
#endif
  
/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/
#ifndef   OS_UART
  #define OS_UART 3
#endif    
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

#define OS_UART_USED (OS_UART == 3)

#define OS_TIMER_RELOAD ((OS_PCLK_TIMER/OS_TICK_FREQ)-1)
#if (OS_TIMER_RELOAD >= 0x10000)
  #error "TPU Timer can not be used, please check configuration"
#endif
	
/*********************************************************************
*
*       SFR definition of sfrs used in RTOSInit.c
*
*       We define sfrs here to be independend of any i/o header file
*/
#define MSTPCRA      (*(volatile OS_U16 *) (0xFFFFFDC8)) // System Control            
#define MSTPCRB      (*(volatile OS_U16 *) (0xFFFFFDCA)) // System Control            
#define IPRF         (*(volatile OS_U16 *) (0xFFFFFD4A)) // interrupt priority reg.G 
#define IPRL         (*(volatile OS_U16 *) (0xFFFFFD56)) // interrupt priority reg.L 
#define INTCR        (*(volatile OS_U8  *) (0xFFFFFF32)) // interrupt CONTROL reg    
#define TPU_TSTR     (*(volatile OS_U8 *)  (0xFFFFFFBC)) // Timer start               
#define TPU_TCR0     (*(volatile OS_U8 *)  (0xFFFFFFC0)) // Timer control register      
#define TPU_TMDR0    (*(volatile OS_U8 *)  (0xFFFFFFC1)) // Timer mode register         
#define TPU_TIOR0    (*(volatile OS_U8 *)  (0xFFFFFFC2)) // Timer I/O ctrl register     
#define TPU_TIER0    (*(volatile OS_U8 *)  (0xFFFFFFC4)) // Timer int enable register   
#define TPU_TISR0    (*(volatile OS_U8 *)  (0xFFFFFFC5)) // Timer status register       
#define TPU_TCNT0    (*(volatile OS_U16*)  (0xFFFFFFC6)) // Timer count reg. register   
#define TPU_TGRA0    (*(volatile OS_U16*)  (0xFFFFFFC8)) // Timer general regA register 

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW
*
*
*  Initialize hardware required for embOS to run.
*  1. We switch CPU into interrupt mode 2 (8 interrupt priorities)
*  2. We initialize a timer for embOS with low interrupt priority.
*     We use TPU Counter 0 in output compare mode.
*     This function should work on any hardware, but may have to be modified,
*     if you decide to use an other timer.   
*/
void OS_InitHW(void) {
  OS_IncDI();                                    // Initially disable interrupts
  //
  // Set Interrupt mode
  // 
  INTCR       &= ~((1 << 5) | (1 << 4));         // Set interrupt mode 0
  INTCR       |= (1 << 5);                       // select interrupt mode 2
  //
  // Init PLL
  //
  SCKCR.BIT.ICK = 0;                             // CPU clock 12MHz*4=48MHz:
  SCKCR.BIT.PCK = 1;                             // Peripheral clock 12MHz*2=24MHz:
  SCKCR.BIT.BCK = 0;                             // BUS clock 12MHz*4=48MHz:
  BSC.IDLCR.BIT.IDLS3 = 0;                       // no IDLE
  BSC.IDLCR.BIT.IDLS2 = 0;                       // no IDLE
  BSC.IDLCR.BIT.IDLS1 = 0;                       // no IDLE
  BSC.IDLCR.BIT.IDLS0 = 0;                       // no IDLE
  BSC.IDLCR.BIT.IDLCA = 0;                       // no IDLE
  BSC.IDLCR.BIT.IDLCB = 0;                       // no IDLE
  // 
  // Configure embOS timer
  //
  MSTPCRA     &= ~(1 << 0);                      // Release TPU stop mode
  TPU_TSTR    &= ~(1 << 0);                      // Stop counter
  TPU_TCR0     = (1 << 5);                       // Prescaler set to 1 (b2..b0: 0), Count on rising edge, Clear TCNT on TRGA compare
  TPU_TIOR0   &= 0xF0;                           // No output for TRGA compare match
  TPU_TGRA0    = OS_TIMER_RELOAD;                // This value is for 1000 Interrupts per sec => 1ms tick
  TPU_TMDR0    = 0x00;                           // Set timer to normal mode
  TPU_TIER0   |= (1 << 0);                       // enable compare match interrupt
  IPRF         = 0x0010;                         // Set timer int to lowest priority (1)
  TPU_TSTR    |= (1 << 0);                       // Start counter
  OS_COM_Init();                                 // Initialize communication to embOSView
  OS_DecRI();                                    // Restore low priority interrupt enable state
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*  Please note:
*  This is basically the "core" of the idle loop
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) { // Idle loop: No task is ready to exec
  for (;;);          // Nothing to do ... wait for an interrupt
}

/*********************************************************************
*
*       OS_GetTimeCycles()
*
*  This routine is required for task-info via embOSView
*  or high precision time measurment functions.
*  It returns the system time in clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 Cnt;
  OS_U32 Time;

  Cnt = TPU_TCNT0;
  Time = OS_Time;
  if (TPU_TISR0 & (1 << 0)) { // Re-read if interrupt pending
    Time++;
    Cnt = TPU_TCNT0;
  }
  return (OS_PCLK_TIMER/1000) * Time + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*
*  Purpose:
*    Convert Cycles into micro seconds.
*
*    If your clock frequency is not a multiple of 1 MHz, you may have to
*    modify this routine in order to get proper diagonstics.
*  Please note:
*    This routine is required for profiling or high resolution time
*    measurement. It does not affect operation of embOS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_TickHandler()
*
*       embOS timer interrupt handler
*/
void OS_ISR_TickHandler(void) {
  TPU_TISR0 &= ~(1 << 0);
  OS_HandleTick();
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
*       embOS timer interrupt handler
*/
__interrupt(vect=88) void OS_ISR_Tick (void) {
  OS_CallNestableISR(&OS_ISR_TickHandler);  
}    

/*********************************************************************
*
*       Communication for embOSView via UART
*
*/
#if OS_UART_USED
#define SCI_SMR3   (*(volatile OS_U8 *) (0xFFFFFE90)) // SCI Serial mode register    
#define SCI_BRR3   (*(volatile OS_U8 *) (0xFFFFFE91)) // SCI Bit rate register       
#define SCI_SCR3   (*(volatile OS_U8 *) (0xFFFFFE92)) // SCI Serial control register 
#define SCI_TDR3   (*(volatile OS_U8 *) (0xFFFFFE93)) // SCI Transmit data register  
#define SCI_SSR3   (*(volatile OS_U8 *) (0xFFFFFE94)) // SCI Serial status register  
#define SCI_RDR3   (*(volatile OS_U8 *) (0xFFFFFE95)) // SCI Receive data register   
#define SCI_SCMR3  (*(volatile OS_U8 *) (0xFFFFFE96)) // SCI Smart card moderegister 
#define P6DDR      (*(volatile OS_U8 *) (0xFFFFFB85)) // Port P6 data direction register   
#define P6ICR      (*(volatile OS_U8 *) (0xFFFFFB95)) // Port P6 inpt buffer ctrl register 

#define BRR_VAL ((OS_PCLK_UART+OS_BAUDRATE*16L)/(OS_BAUDRATE*32L) -1 )
#if BRR_VAL > 0xFF
  #error "UART can not be used with standard intialization."  
#endif

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  MSTPCRB   &=  ~(1 << 12);                  // Enable SIO module (clear stop mode) 
  SCI_SCR3  &= ~((1 << 5)                    // Clear Tx enable
                |(1 << 4));                  // Clear Rx enable
  SCI_SCR3   =   (0 << 7)                    // Clear Tx interrupt enable
                |(0 << 6)                    // Clear Rx interrupt enable
                |(0 << 5)                    // Clear Tx enable
                |(0 << 4)                    // Clear Rx enable
                |(0 << 3)                    // Clear multi processor mode
                |(0 << 2)                    // Clear Tx end interrupt enable
                |(0 << 1)                    // Set internal clock mode
                |(0 << 0);                   // SCLK output not used
  SCI_SMR3   =   (0 << 7)                    // Select Async. mode
                |(0 << 6)                    // Select 8 data bits
                |(0 << 5)                    // Disable parity
                |(0 << 4)                    // Even parity (don't care)
                |(0 << 3)                    // Select 1 stop bit
                |(0 << 2)                    // Disable multi processor mode
                |(0 << 1)                    // Clock divisor 1
                |(0 << 0);                   // Clock divisor 1
  SCI_BRR3   =  BRR_VAL;                     // Set calculated divisor
  IPRL       =  0x0010;                      // Set UART int to lowest priority (1)
  SCI_SCR3  |=   (1 << 6)                    // Enable Rx interrupt
                |(1 << 5)                    // Enable Tx
                |(1 << 4);                   // Enable Rx
  P6DDR      = 0x01;                         // P6DDR bit 0 is TxD, bit 1 is RxD, bit 2 is SCK write only 
  P6ICR      = 0x02;                         // P6ICR set to 1 to enable input for RxD 
}

/*********************************************************************
*
*       OS_COM_Send1() for UART0: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  SCI_TDR3  = c;
  SCI_SSR3 &= ~(1 << 2);
  SCI_SCR3 |= (1 << 2);                      //  Enable TxEnd-Interrupt  
}

/*********************************************************************
*
*       OS_ISR_Tx_Handler() 
*
*       Uart Tx Interrupt Handler
*/
void OS_ISR_Tx_Handler(void) {
  SCI_SSR3 &= ~(1 << 2);
  SCI_SSR3 &= ~(1 << 7);
  if (OS_OnTx()) {                           //  No more characters to send ?
    SCI_SCR3 &= ~(1 << 2);                   //  Disable TxEnd Interrupt
  }
}

/*********************************************************************
*
*       OS_ISR_tx() 
*      
*       ISR for uart tx
*/
__interrupt(vect=163) void OS_ISR_tx (void) {
  OS_CallNestableISR(&OS_ISR_Tx_Handler);  
}

/*********************************************************************
*
*       OS_ISR_Rx_Handler()
*
*       Uart Rx Interrupt Handler
*/
void OS_ISR_Rx_Handler(void) {
  SCI_SSR3 &= ~(1 << 6);
  OS_OnRx(SCI_RDR3);  
}

/*********************************************************************
*
*       OS_ISR_rx() 
*
*       ISR for uart rx 
*/
__interrupt(vect=161) void OS_ISR_rx (void) {
  OS_CallNestableISR(&OS_ISR_Rx_Handler);  
}

/*********************************************************************
*
*       OS_ISR_Err_Handler()
*
*       Uart Error Interrupt Handler
*/
void OS_ISR_Err_Handler(void) {
  SCI_SSR3 &= ~((1 << 5) | (1 << 4) | (1 << 3));  // Clear all errors
}

/*********************************************************************
*
*       OS_ISR_err() 
*
*       ISR for uart error interrupt
*/
__interrupt(vect=160) void OS_ISR_err (void) {
  OS_CallNestableISR(&OS_ISR_Err_Handler);
}

/*********************************************************************
*
*       Communication for embOSView not selected
*
*/
#else  /* No communication routines required, use dummies  */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }
#endif /* OS_UART_USED */

/***** End of File **************************************************/
